package com.supervision.pdfqaserver.dto;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.supervision.pdfqaserver.domain.ChineseEnglishWords;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

import java.util.*;

/**
 * 实体关系抽取
 */
@Slf4j
@Data
public class EREDTO {

    private List<EntityExtractionDTO> entities = new ArrayList<>();

    private List<RelationExtractionDTO> relations = new ArrayList<>();

    public EREDTO() {
    }

    public static EREDTO fromTextJson(String json,String truncationId) {
        EREDTO eredto = new EREDTO();
        JSONObject jsonObject = JSONObject.parseObject(json);
        JSONArray nodes = jsonObject.getJSONArray("nodes");
        JSONArray relations = jsonObject.getJSONArray("relations");
        List<EntityExtractionDTO> entities = new ArrayList<>();
        List<RelationExtractionDTO> relationsList = new ArrayList<>();
        if (CollUtil.isNotEmpty(nodes)){
            for (Object node : nodes) {
                JSONObject nodeJson = (JSONObject) node;
                String name = nodeJson.getString("name");
                String type = nodeJson.getString("type");
                JSONObject attributes = nodeJson.getJSONObject("attributes");
                if (CollUtil.isNotEmpty(attributes)){
                    List<ERAttributeDTO> erAttributeDTOS = new ArrayList<>();
                    for (String key : attributes.keySet()) {
                        Object value = attributes.get(key);
                        String valueString = attributes.getString(key);
                        ERAttributeDTO erAttributeDTO = new ERAttributeDTO(key, valueString, value instanceof Number?"1":"0");
                        erAttributeDTOS.add(erAttributeDTO);
                    }
                    EntityExtractionDTO entityExtraction = new EntityExtractionDTO(truncationId,name,type, erAttributeDTOS);
                    entities.add(entityExtraction);
                }
            }
        }
        if (CollUtil.isNotEmpty(relations)){
            for (Object relation : relations) {
                JSONObject relationJson = (JSONObject) relation;
                String source = relationJson.getString("source");
                String target = relationJson.getString("target");
                String type = relationJson.getString("type");
                JSONObject attributes = relationJson.getJSONObject("attributes");
                if (CollUtil.isNotEmpty(attributes)){
                    List<ERAttributeDTO> erAttributeDTOS = new ArrayList<>();
                    for (String key : attributes.keySet()) {
                        Object value = attributes.get(key);
                        String valueString = attributes.getString(key);
                        ERAttributeDTO erAttributeDTO = new ERAttributeDTO(key, valueString, value instanceof Number?"1":"0");
                        erAttributeDTOS.add(erAttributeDTO);
                    }
                    if (StrUtil.isEmpty(source) || StrUtil.isEmpty(target)){
                        log.warn("truncationId:{} relation:{} 关系中source or target is empty",truncationId,relationJson);
                        continue;
                    }
                    Optional<EntityExtractionDTO> sourceTypeOpt = entities.stream().filter(e -> StrUtil.equals(e.getEntity(), source)).findFirst();
                    if (sourceTypeOpt.isEmpty()){
                        log.warn("truncationId:{} relation:{} 关系中source在实体中不存在",truncationId,relationJson);
                        continue;
                    }
                    Optional<EntityExtractionDTO> targetTypeOpt = entities.stream().filter(e -> StrUtil.equals(e.getEntity(), target)).findFirst();
                    if (targetTypeOpt.isEmpty()){
                        log.warn("truncationId:{} relation:{} 关系中target在实体中不存在",truncationId,relationJson);
                        continue;
                    }
                    RelationExtractionDTO relationExtractionDTO = new RelationExtractionDTO(truncationId,source,
                            sourceTypeOpt.get().getEntity(),type,target,targetTypeOpt.get().getEntity(), erAttributeDTOS);
                    relationsList.add(relationExtractionDTO);
                }
            }
        }
        eredto.setEntities(entities);
        eredto.setRelations(relationsList);
        return eredto;
    }

    public static EREDTO fromTableJson(String json,String truncationId) {

        EREDTO eredto = new EREDTO();
        JSONObject jsonObject = JSONObject.parseObject(json);
        JSONArray tables = jsonObject.getJSONArray("table_data");

        if (CollUtil.isEmpty(tables)){
            return eredto;
        }
       List<EntityExtractionDTO> entities = new ArrayList<>();
        for (Object table : tables) {
            JSONObject tableJson = (JSONObject) table;
            if (CollUtil.isEmpty(tableJson)){
                continue;
            }
            EntityExtractionDTO entityExtractionDTO = new EntityExtractionDTO();
            entityExtractionDTO.setEntity("row");
            entityExtractionDTO.setName("row");
            entityExtractionDTO.setTruncationId(truncationId);
            List<ERAttributeDTO> erAttributeDTOS = new ArrayList<>();
            for (Map.Entry<String, Object> tableEntry : tableJson.entrySet()) {
                String key = tableEntry.getKey();
                Object value = tableEntry.getValue();
                ERAttributeDTO erAttributeDTO = new ERAttributeDTO(key, value.toString(), value instanceof Number ? "1" : "0");
                erAttributeDTOS.add(erAttributeDTO);
            }
            entityExtractionDTO.setAttributes(erAttributeDTOS);
            entities.add(entityExtractionDTO);
        }
        eredto.setEntities(entities);
        return eredto;
    }

    public void setEn(List<ChineseEnglishWords> wordsList) {
        if (CollUtil.isEmpty(wordsList)){
            return;
        }
        for (EntityExtractionDTO entity : entities) {
            String entityName = entity.getEntity();
            Optional<ChineseEnglishWords> first = wordsList.stream().filter(w -> StrUtil.equals(w.getChineseWord(), entityName)).findFirst();
            first.ifPresent(chineseEnglishWords -> entity.setEntityEn(chineseEnglishWords.getEnglishWord()));
        }
        for (RelationExtractionDTO relation : relations) {
            String relationName = relation.getRelation();
            Optional<ChineseEnglishWords> first = wordsList.stream().filter(w -> StrUtil.equals(w.getChineseWord(), relationName)).findFirst();
            first.ifPresent(chineseEnglishWords -> relation.setRelationEn(chineseEnglishWords.getEnglishWord()));
        }

    }
}
